@@ -1,6 +1,5 @@
 ##
-# Copyright 2011-2012 Ghent University
-# Copyright 2011-2012 Jens Timmerman
+# Copyright 2011-2014 Ghent University
 #
 # This file is part of EasyBuild,
 # originally created by the HPC team of Ghent University (http://ugent.be/hpc/en),
@@ -24,7 +23,9 @@
 # along with EasyBuild.  If not, see <http://www.gnu.org/licenses/>.
 ##
 """
-This initializes EasyBuild.
+Declares EasyBuild namespace, in an extendable way.
+
+@author: Jens Timmerman (Ghent University)
 """
 from pkgutil import extend_path
 
