@@ -34,7 +34,7 @@ This is the original pure python code, to be exec'ed rather then parsed
 import os
 import re
 import tempfile
-from vsc import fancylogger
+from vsc.utils import fancylogger
 
 from easybuild.framework.easyconfig.format.format import FORMAT_DEFAULT_VERSION, get_format_version
 from easybuild.framework.easyconfig.format.pyheaderconfigobj import EasyConfigFormatConfigObj
