@@ -43,11 +43,13 @@ from distutils.version import StrictVersion
 from subprocess import PIPE
 from vsc.utils import fancylogger
 from vsc.utils.missing import get_subclasses, any
+from vsc.utils.patterns import Singleton
 
 from easybuild.tools.build_log import EasyBuildError
 from easybuild.tools.config import build_option, get_modules_tool, install_path
+from easybuild.tools.environment import modify_env
 from easybuild.tools.filetools import convert_name, mkdir, read_file, which
-from easybuild.tools.module_generator import det_full_module_name, DEVEL_MODULE_SUFFIX, GENERAL_CLASS
+from easybuild.tools.module_naming_scheme import DEVEL_MODULE_SUFFIX
 from easybuild.tools.run import run_cmd
 from easybuild.tools.toolchain import DUMMY_TOOLCHAIN_NAME, DUMMY_TOOLCHAIN_VERSION
 from vsc.utils.missing import nub
@@ -133,6 +135,8 @@ class ModulesTool(object):
     # the regexp, should have a "version" group (multiline search)
     VERSION_REGEXP = None
 
+    __metaclass__ = Singleton
+
     def __init__(self, mod_paths=None):
         """
         Create a ModulesTool object
@@ -141,17 +145,13 @@ class ModulesTool(object):
         """
 
         self.log = fancylogger.getLogger(self.__class__.__name__, fname=False)
-        # make sure we don't have the same path twice
-        if mod_paths is None:
-            self.mod_paths = None
-        else:
-            self.mod_paths = nub(mod_paths)
+        self.mod_paths = None
+        if mod_paths is not None:
+            self.set_mod_paths(mod_paths)
 
         # DEPRECATED!
         self._modules = []
 
-        self.check_module_path()
-
         # actual module command (i.e., not the 'module' wrapper function, but the binary)
         self.cmd = self.COMMAND
         if self.COMMAND_ENVIRONMENT is not None and self.COMMAND_ENVIRONMENT in os.environ:
@@ -168,9 +168,9 @@ class ModulesTool(object):
 
         # some initialisation/verification
         self.check_cmd_avail()
+        self.check_module_path()
         self.check_module_function(allow_mismatch=build_option('allow_modules_tool_mismatch'))
         self.set_and_check_version()
-        self.use_module_paths()
 
         # this can/should be set to True during testing
         self.testing = False
@@ -187,7 +187,6 @@ class ModulesTool(object):
 
     def set_and_check_version(self):
         """Get the module version, and check any requirements"""
-        txt = self.run_module(self.VERSION_OPTION, return_output=True)
         if self.VERSION_REGEXP is None:
             self.log.error('No VERSION_REGEXP defined')
 
@@ -249,41 +248,68 @@ class ModulesTool(object):
             # module function may not be defined (weird, but fine)
             self.log.warning("No 'module' function defined, can't check if it matches %s." % mod_details)
 
+    def set_mod_paths(self, mod_paths=None):
+        """Set mod_paths, based on $MODULEPATH unless a list of module paths is specified."""
+        # make sure we don't have the same path twice, using nub
+        if mod_paths is not None:
+            self.mod_paths = nub(mod_paths)
+            for mod_path in self.mod_paths:
+                self.prepend_module_path(mod_path)
+        else:
+            # no paths specified, so grab list of (existing) module paths from $MODULEPATH
+            self.mod_paths = [p for p in nub(curr_module_paths()) if os.path.exists(p)]
+        self.log.debug("$MODULEPATH after set_mod_paths: %s" % os.environ.get('MODULEPATH', ''))
+
+    def use(self, path):
+        """Add module path via 'module use'."""
+        # make sure path exists before we add it
+        mkdir(path, parents=True)
+        self.run_module(['use', path])
+
+    def unuse(self, path):
+        """Remove module path via 'module unuse'."""
+        self.run_module(['unuse', path])
+
+    def add_module_path(self, path):
+        """Add specified module path (using 'module use') if it's not there yet."""
+        if not path in self.mod_paths:
+            # add module path via 'module use' and make sure self.mod_paths is synced
+            self.use(path)
+            self.set_mod_paths()
+
+    def remove_module_path(self, path):
+        """Remove specified module path (using 'module unuse')."""
+        # remove module path via 'module unuse' and make sure self.mod_paths is synced
+        self.unuse(path)
+        self.set_mod_paths()
+
+    def prepend_module_path(self, path):
+        """Prepend given module path to list of module paths, or bump it to 1st place."""
+        # generic approach: remove the path first (if it's there), then add it again (to the front)
+        self.remove_module_path(path)
+        self.add_module_path(path)
+
     def check_module_path(self):
         """
         Check if MODULEPATH is set and change it if necessary.
         """
-        # if self.mod_paths is not specified, use $MODULEPATH and make sure the EasyBuild module path is in there (first)
+        # if self.mod_paths is not specified, define it and make sure the EasyBuild module path is in there (first)
         if self.mod_paths is None:
-            # take module path from environment
-            self.mod_paths = [x for x in nub(os.environ.get('MODULEPATH', '').split(':')) if len(x) > 0]
+            # take (unique) module paths from environment
+            self.set_mod_paths()
             self.log.debug("self.mod_paths set based on $MODULEPATH: %s" % self.mod_paths)
 
             # determine module path for EasyBuild install path to be included in $MODULEPATH
-            eb_modpath = os.path.join(install_path(typ='modules'), GENERAL_CLASS)
+            eb_modpath = os.path.join(install_path(typ='modules'), build_option('suffix_modules_path'))
 
             # make sure EasyBuild module path is in 1st place
-            self.mod_paths = [x for x in self.mod_paths if not x == eb_modpath]
-            self.mod_paths.insert(0, eb_modpath)
+            self.prepend_module_path(eb_modpath)
             self.log.info("Prepended list of module paths with path used by EasyBuild: %s" % eb_modpath)
 
         # set the module path environment accordingly
-        os.environ['MODULEPATH'] = ':'.join(self.mod_paths)
-        self.log.info("$MODULEPATH set based on list of module paths: %s" % os.environ['MODULEPATH'])
-
-    def use_module_paths(self):
-        """Run 'module use' on all paths in $MODULEPATH."""
-        # we need to run '<module command> python use <path>' on all paths in $MODULEPATH
-        # not all modules tools follow whatever is in $MODULEPATH, some take extra action for every module path
-        # usually, additional environment variables are set, e.g. $LMOD_DEFAULT_MODULEPATH or $MODULEPATH_modshare
-        # note: we're stepping through the mod_paths in reverse order to preserve order in $MODULEPATH in the end
-        for modpath in self.mod_paths[::-1]:
-            if not os.path.isabs(modpath):
-                modpath = os.path.join(os.getcwd(), modpath)
-            if os.path.exists(modpath):
-                self.run_module(['use', modpath])
-            else:
-                self.log.warning("Ignoring non-existing module path in $MODULEPATH: %s" % modpath)
+        for mod_path in self.mod_paths[::-1]:
+            self.use(mod_path)
+        self.log.info("$MODULEPATH set based on list of module paths (via 'module use'): %s" % os.environ['MODULEPATH'])
 
     def available(self, mod_name=None):
         """
@@ -309,57 +335,31 @@ class ModulesTool(object):
         """
         return mod_name in self.available(mod_name)
 
-    def add_module(self, modules):
-        """
-        Check if module exist, if so add to list.
-        """
-        self.log.deprecated("Use of add_module function should be replaced by load([<list of modules>])", '2.0')
-        for mod in modules:
-            if isinstance(mod, (list, tuple)):
-                mod_dict = {
-                    'name': mod[0],
-                    'version': mod[1],
-                    'versionsuffix': '',
-                    'toolchain': {
-                        'name': DUMMY_TOOLCHAIN_NAME,
-                        'version': DUMMY_TOOLCHAIN_VERSION,
-                    },
-                }
-                mod_name = det_full_module_name(mod_dict)
-            elif isinstance(mod, basestring):
-                mod_name = mod
-            elif isinstance(mod, dict):
-                mod_name = det_full_module_name(mod)
-            else:
-                self.log.error("Can't add module %s: unknown type" % str(mod))
-
-            mods = self.available(mod_name)
-            if mod_name in mods:
-                # ok
-                self._modules.append(mod_name)
-            else:
-                if len(mods) == 0:
-                    self.log.warning('No module %s available' % str(mod))
-                else:
-                    self.log.warning('More than one module found for %s: %s' % (mod, mods))
-                continue
-
-    def remove_module(self, modules):
-        """
-        Remove modules from list.
-        """
-        self.log.deprecated("remove_module should no longer be used (add_module is deprecated too).", '2.0')
-        for mod in modules:
-            self._modules = [m for m in self._modules if not m == mod]
-
-    def load(self, modules=None):
+    def load(self, modules, mod_paths=None, purge=False, orig_env=None):
         """
         Load all requested modules.
+
+        @param modules: list of modules to load
+        @param mod_paths: list of module paths to activate before loading
+        @param purge: whether or not a 'module purge' should be run before loading
+        @param orig_env: original environment to restore after running 'module purge'
         """
-        if modules is None:
-            # deprecated behavior if no modules were passed by argument
-            self.log.deprecated("Loading modules listed in _modules class variable", '2.0')
-            modules = self._modules[:]
+        if mod_paths is None:
+            mod_paths = []
+
+        # purge all loaded modules if desired
+        if purge:
+            self.purge()
+            # restore original environment if provided
+            if orig_env is not None:
+                modify_env(os.environ, orig_env)
+
+        # make sure $MODULEPATH is set correctly after purging
+        self.check_module_path()
+        # extend $MODULEPATH if needed
+        for mod_path in mod_paths:
+            full_mod_path = os.path.join(install_path('mod'), build_option('suffix_modules_path'), mod_path)
+            self.prepend_module_path(full_mod_path)
 
         for mod in modules:
             self.run_module('load', mod)
@@ -445,9 +445,7 @@ class ModulesTool(object):
             os.environ['MODULEPATH'] = kwargs[module_path_key]
             self.log.deprecated("Use of '%s' named argument in 'run_module'" % module_path_key, '2.0')
 
-        # after changing $MODULEPATH, we should adjust self.mod_paths and run use_module_paths(),
-        # but we can't do that here becaue it would yield infinite recursion on run_module
-        self.log.debug('Current MODULEPATH: %s' % os.environ['MODULEPATH'])
+        self.log.debug('Current MODULEPATH: %s' % os.environ.get('MODULEPATH', ''))
 
         # change our LD_LIBRARY_PATH here
         environ = os.environ.copy()
@@ -464,16 +462,16 @@ class ModulesTool(object):
                 self.log.error(msg % (type(self.COMMAND_SHELL), self.COMMAND_SHELL))
             cmdlist = self.COMMAND_SHELL + cmdlist
 
-        self.log.debug("Running module command '%s' from %s" % (' '.join(cmdlist + args), os.getcwd()))
+        full_cmd = ' '.join(cmdlist + args)
+        self.log.debug("Running module command '%s' from %s" % (full_cmd, os.getcwd()))
         proc = subprocess.Popen(cmdlist + args, stdout=PIPE, stderr=PIPE, env=environ)
         # stdout will contain python code (to change environment etc)
         # stderr will contain text (just like the normal module command)
         (stdout, stderr) = proc.communicate()
+        self.log.debug("Output of module command '%s': stdout: %s; stderr: %s" % (full_cmd, stdout, stderr))
         if original_module_path is not None:
             os.environ['MODULEPATH'] = original_module_path
             self.log.deprecated("Restoring $MODULEPATH back to what it was before running module command/.", '2.0')
-            # after changing $MODULEPATH, we should adjust self.mod_paths and run use_module_paths(),
-            # but we can't do that here becaue it would yield infinite recursion on run_module
 
         if kwargs.get('return_output', False):
             return stdout + stderr
@@ -569,6 +567,7 @@ class ModulesTool(object):
 class EnvironmentModulesC(ModulesTool):
     """Interface to (C) environment modules (modulecmd)."""
     COMMAND = "modulecmd"
+    REQ_VERSION = '3.2.10'
     VERSION_REGEXP = r'^\s*(VERSION\s*=\s*)?(?P<version>\d\S*)\s*'
 
     def module_software_name(self, mod_name):
@@ -591,6 +590,7 @@ class EnvironmentModulesTcl(EnvironmentModulesC):
     # older versions of modulecmd.tcl don't have a decent hashbang, so we run it under a tclsh shell
     COMMAND_SHELL = ['tclsh']
     VERSION_OPTION = ''
+    REQ_VERSION = None
     VERSION_REGEXP = r'^Modules\s+Release\s+Tcl\s+(?P<version>\d\S*)\s'
 
     def set_ld_library_path(self, ld_library_paths):
@@ -617,7 +617,7 @@ class EnvironmentModulesTcl(EnvironmentModulesC):
 
         tweak_stdout_fn = None
         # for 'active' module (sub)commands that yield changes in environment, we need to tweak stdout before exec'ing
-        if args[0] in ['load', 'purge', 'unload', 'use']:
+        if args[0] in ['load', 'purge', 'unload', 'use', 'unuse']:
             tweak_stdout_fn = tweak_stdout
         kwargs.update({'tweak_stdout': tweak_stdout_fn})
 
@@ -637,33 +637,34 @@ class EnvironmentModulesTcl(EnvironmentModulesC):
 
         return clean_mods
 
+    def remove_module_path(self, path):
+        """Remove specified module path (using 'module unuse')."""
+        # remove module path via 'module use' and make sure self.mod_paths is synced
+        # modulecmd.tcl keeps track of how often a path was added via 'module use',
+        # so we need to check to make sure it's really removed
+        while path in self.mod_paths:
+            self.unuse(path)
+            self.set_mod_paths()
+
 
 class Lmod(ModulesTool):
     """Interface to Lmod."""
     COMMAND = 'lmod'
     COMMAND_ENVIRONMENT = 'LMOD_CMD'
     # required and optimal version
-    # we need at least Lmod v5.2 (and it can't be a release candidate)
-    REQ_VERSION = '5.2'
+    # we need at least Lmod v5.6.3 (and it can't be a release candidate)
+    REQ_VERSION = '5.6.3'
     VERSION_REGEXP = r"^Modules\s+based\s+on\s+Lua:\s+Version\s+(?P<version>\d\S*)\s"
 
     def __init__(self, *args, **kwargs):
         """Constructor, set lmod-specific class variable values."""
-        # $LMOD_EXPERT needs to be set to avoid EasyBuild tripping over fiddly bits in output
-        os.environ['LMOD_EXPERT'] = '1'
+        # $LMOD_QUIET needs to be set to avoid EasyBuild tripping over fiddly bits in output
+        os.environ['LMOD_QUIET'] = '1'
         # make sure Lmod ignores the spider cache ($LMOD_IGNORE_CACHE supported since Lmod 5.2)
         os.environ['LMOD_IGNORE_CACHE'] = '1'
 
         super(Lmod, self).__init__(*args, **kwargs)
 
-    def set_and_check_version(self):
-        """Get the module version, and check any requirements"""
-
-        # 'lmod python update' needs to be run after changing $MODULEPATH
-        self.run_module('update')
-
-        super(Lmod, self).set_and_check_version()
-
     def check_module_function(self, *args, **kwargs):
         """Check whether selected module tool matches 'module' function definition."""
         if not 'regex' in kwargs:
@@ -692,6 +693,7 @@ class Lmod(ModulesTool):
         spider_cmd = os.path.join(os.path.dirname(self.cmd), 'spider')
         cmd = [spider_cmd, '-o', 'moduleT', os.environ['MODULEPATH']]
         self.log.debug("Running command '%s'..." % ' '.join(cmd))
+
         proc = subprocess.Popen(cmd, stdout=PIPE, stderr=PIPE, env=os.environ)
         (stdout, stderr) = proc.communicate()
 
@@ -720,6 +722,11 @@ class Lmod(ModulesTool):
         name_re = re.compile('^conflict\("*(?P<name>[^ "]+)"\).*$', re.M)
         return self.get_value_from_modulefile(mod_name, name_re)
 
+    def prepend_module_path(self, path):
+        # Lmod pushes a path to the front on 'module use'
+        self.use(path)
+        self.set_mod_paths()
+
 
 def get_software_root_env_var_name(name):
     """Return name of environment variable for software root."""
