@@ -32,11 +32,11 @@ import os
 import re
 import shutil
 import tempfile
-from test.framework.utilities import EnhancedTestCase
+from test.framework.utilities import EnhancedTestCase, init_config
 from unittest import TestLoader, main
 
 import easybuild.tools.modules as modules
-from easybuild.framework.easyconfig.easyconfig import EasyConfig
+from easybuild.framework.easyconfig.easyconfig import EasyConfig, ActiveMNS
 from easybuild.tools.toolchain.utilities import search_toolchain
 from test.framework.utilities import find_full_path
 
@@ -54,6 +54,13 @@ class ToolchainTest(EnhancedTestCase):
         self.orig_modpath = os.environ.get('MODULEPATH', '')
         os.environ['MODULEPATH'] = find_full_path(os.path.join('test', 'framework', 'modules'))
 
+    def get_toolchain(self, name, version=None):
+        """Get a toolchain object instance to test with."""
+        tc_class, _ = search_toolchain(name)
+        self.assertEqual(tc_class.NAME, name)
+        tc = tc_class(version=version, mns=ActiveMNS())
+        return tc
+
     def test_toolchain(self):
         """Test whether toolchain is initialized correctly."""
         ec_file = find_full_path(os.path.join('test', 'framework', 'easyconfigs', 'gzip-1.4.eb'))
@@ -69,15 +76,11 @@ class ToolchainTest(EnhancedTestCase):
 
     def test_goalf_toolchain(self):
         """Test for goalf toolchain."""
-        name = "goalf"
-        tc, _ = search_toolchain(name)
-        self.assertEqual(tc.NAME, name)
-        self.tc = tc(version="1.1.0-no-OFED")
+        self.get_toolchain("goalf", version="1.1.0-no-OFED")
 
     def test_get_variable_compilers(self):
         """Test get_variable function to obtain compiler variables."""
-        tc_class, _ = search_toolchain("goalf")
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.prepare()
 
         cc = tc.get_variable('CC')
@@ -108,8 +111,7 @@ class ToolchainTest(EnhancedTestCase):
 
     def test_get_variable_mpi_compilers(self):
         """Test get_variable function to obtain compiler variables."""
-        tc_class, _ = search_toolchain("goalf")
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.set_options({'usempi': True})
         tc.prepare()
 
@@ -142,8 +144,7 @@ class ToolchainTest(EnhancedTestCase):
 
     def test_get_variable_seq_compilers(self):
         """Test get_variable function to obtain compiler variables."""
-        tc_class, _ = search_toolchain("goalf")
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.set_options({'usempi': True})
         tc.prepare()
 
@@ -158,8 +159,7 @@ class ToolchainTest(EnhancedTestCase):
 
     def test_get_variable_libs_list(self):
         """Test get_variable function to obtain list of libraries."""
-        tc_class, _ = search_toolchain("goalf")
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.prepare()
 
         ldflags = tc.get_variable('LDFLAGS', typ=list)
@@ -172,8 +172,7 @@ class ToolchainTest(EnhancedTestCase):
         Check that elements of variables are passed by value, not by reference,
         which is required to ensure correctness.
         """
-        tc_class, _ = search_toolchain("goalf")
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.prepare()
 
         pass_by_value = True
@@ -195,10 +194,9 @@ class ToolchainTest(EnhancedTestCase):
         """Test whether optimization flags are being set correctly."""
 
         flag_vars = ['CFLAGS', 'CXXFLAGS', 'FFLAGS', 'F90FLAGS']
-        tc_class, _ = search_toolchain("goalf")
 
         # check default optimization flag (e.g. -O2)
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.set_options({})
         tc.prepare()
         for var in flag_vars:
@@ -207,7 +205,7 @@ class ToolchainTest(EnhancedTestCase):
 
         # check other optimization flags
         for opt in ['noopt', 'lowopt', 'opt']:
-            tc = tc_class(version="1.1.0-no-OFED")
+            tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
             for enable in [True, False]:
                 tc.set_options({opt: enable})
                 tc.prepare()
@@ -222,11 +220,10 @@ class ToolchainTest(EnhancedTestCase):
         """Test whether combining optimization levels works as expected."""
 
         flag_vars = ['CFLAGS', 'CXXFLAGS', 'FFLAGS', 'F90FLAGS']
-        tc_class, _ = search_toolchain("goalf")
 
         # check combining of optimization flags (doesn't make much sense)
         # lowest optimization should always be picked
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.set_options({'lowopt': True, 'opt':True})
         tc.prepare()
         for var in flag_vars:
@@ -234,7 +231,7 @@ class ToolchainTest(EnhancedTestCase):
             flag = '-%s' % tc.COMPILER_SHARED_OPTION_MAP['lowopt']
             self.assertTrue(flag in flags)
 
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.set_options({'noopt': True, 'lowopt':True})
         tc.prepare()
         for var in flag_vars:
@@ -242,7 +239,7 @@ class ToolchainTest(EnhancedTestCase):
             flag = '-%s' % tc.COMPILER_SHARED_OPTION_MAP['noopt']
             self.assertTrue(flag in flags)
 
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.set_options({'noopt':True, 'lowopt': True, 'opt':True})
         tc.prepare()
         for var in flag_vars:
@@ -254,12 +251,11 @@ class ToolchainTest(EnhancedTestCase):
         """Test whether shared compiler flags are set correctly."""
 
         flag_vars = ['CFLAGS', 'CXXFLAGS', 'FFLAGS', 'F90FLAGS']
-        tc_class, _ = search_toolchain("goalf")
 
         # setting option should result in corresponding flag to be set (shared options)
         for opt in ['pic', 'verbose', 'debug', 'static', 'shared']:
             for enable in [True, False]:
-                tc = tc_class(version="1.1.0-no-OFED")
+                tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
                 tc.set_options({opt: enable})
                 tc.prepare()
                 # we need to make sure we check for flags, not letter (e.g. 'v' vs '-v')
@@ -275,12 +271,11 @@ class ToolchainTest(EnhancedTestCase):
         """Test whether unique compiler flags are set correctly."""
 
         flag_vars = ['CFLAGS', 'CXXFLAGS', 'FFLAGS', 'F90FLAGS']
-        tc_class, _ = search_toolchain("goalf")
 
         # setting option should result in corresponding flag to be set (unique options)
         for opt in ['unroll', 'optarch', 'openmp']:
             for enable in [True, False]:
-                tc = tc_class(version="1.1.0-no-OFED")
+                tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
                 tc.set_options({opt: enable})
                 tc.prepare()
                 flag = '-%s' % tc.COMPILER_UNIQUE_OPTION_MAP[opt]
@@ -291,16 +286,38 @@ class ToolchainTest(EnhancedTestCase):
                     else:
                         self.assertTrue(flag not in flags, "%s: False means no %s in %s" % (opt, flag, flags))
 
+    def test_override_optarch(self):
+        """Test whether overriding the optarch flag works."""
+        flag_vars = ['CFLAGS', 'CXXFLAGS', 'FFLAGS', 'F90FLAGS']
+        for optarch_var in ['march=lovelylovelysandybridge', None]:
+            build_options = {'optarch': optarch_var}
+            init_config(build_options=build_options)
+            for enable in [True, False]:
+                tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
+                tc.set_options({'optarch': enable})
+                tc.prepare()
+                flag = None
+                if optarch_var is not None:
+                    flag = '-%s' % optarch_var
+                else:
+                    flag = '-march=native'
+
+                for var in flag_vars:
+                    flags = tc.get_variable(var)
+                    if enable:
+                        self.assertTrue(flag in flags, "optarch: True means %s in %s" % (flag, flags))
+                    else:
+                        self.assertFalse(flag in flags, "optarch: False means no %s in %s" % (flag, flags))
+
     def test_misc_flags_unique_fortran(self):
         """Test whether unique Fortran compiler flags are set correctly."""
 
         flag_vars = ['FFLAGS', 'F90FLAGS']
-        tc_class, _ = search_toolchain("goalf")
 
         # setting option should result in corresponding flag to be set (Fortran unique options)
         for opt in ['i8', 'r8']:
             for enable in [True, False]:
-                tc = tc_class(version="1.1.0-no-OFED")
+                tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
                 tc.set_options({opt: enable})
                 tc.prepare()
                 flag = '-%s' % tc.COMPILER_UNIQUE_OPTION_MAP[opt]
@@ -315,10 +332,9 @@ class ToolchainTest(EnhancedTestCase):
         """Test whether precision flags are being set correctly."""
 
         flag_vars = ['CFLAGS', 'CXXFLAGS', 'FFLAGS', 'F90FLAGS']
-        tc_class, _ = search_toolchain("goalf")
 
         # check default precision flag
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.prepare()
         for var in flag_vars:
             flags = tc.get_variable(var)
@@ -328,7 +344,7 @@ class ToolchainTest(EnhancedTestCase):
         # check other precision flags
         for opt in ['strict', 'precise', 'loose', 'veryloose']:
             for enable in [True, False]:
-                tc = tc_class(version="1.1.0-no-OFED")
+                tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
                 tc.set_options({opt: enable})
                 tc.prepare()
                 val = ' '.join(['-%s' % f for f in tc.COMPILER_UNIQUE_OPTION_MAP[opt]])
@@ -341,10 +357,7 @@ class ToolchainTest(EnhancedTestCase):
 
     def test_cgoolf_toolchain(self):
         """Test for cgoolf toolchain."""
-        name = "cgoolf"
-        tc_class, _ = search_toolchain(name)
-        self.assertEqual(tc_class.NAME, name)
-        tc = tc_class(version="1.1.6")
+        tc = self.get_toolchain("cgoolf", version="1.1.6")
         tc.prepare()
 
         self.assertEqual(tc.get_variable('CC'), 'clang')
@@ -354,18 +367,13 @@ class ToolchainTest(EnhancedTestCase):
 
     def test_comp_family(self):
         """Test determining compiler family."""
-
-        tc_class, _ = search_toolchain("goalf")
-        tc = tc_class(version="1.1.0-no-OFED")
+        tc = self.get_toolchain("goalf", version="1.1.0-no-OFED")
         tc.prepare()
-
         self.assertEqual(tc.comp_family(), "GCC")
 
     def test_goolfc(self):
         """Test whether goolfc is handled properly."""
-
-        tc_class, _ = search_toolchain("goolfc")
-        tc = tc_class(version="1.3.12")
+        tc = self.get_toolchain("goolfc", version="1.3.12")
         opts = {'cuda_gencode': ['arch=compute_35,code=sm_35', 'arch=compute_10,code=compute_10']}
         tc.set_options(opts)
         tc.prepare()
@@ -411,10 +419,7 @@ class ToolchainTest(EnhancedTestCase):
             for fftlib in fftw_libs:
                 open(os.path.join(tmpdir, subdir, 'lib%s.a' % fftlib), 'w').write('foo')
 
-        name = "ictce"
-        tc_class, _ = search_toolchain(name)
-        self.assertEqual(tc_class.NAME, name)
-        tc = tc_class(version="4.1.13")
+        tc = self.get_toolchain("ictce", version="4.1.13")
         tc.prepare()
 
         self.assertEqual(tc.get_variable('CC'), 'icc')
@@ -422,7 +427,7 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(tc.get_variable('F77'), 'ifort')
         self.assertEqual(tc.get_variable('F90'), 'ifort')
 
-        tc = tc_class(version="4.1.13")
+        tc = self.get_toolchain("ictce", version="4.1.13")
         opts = {'usempi': True}
         tc.set_options(opts)
         tc.prepare()
@@ -436,7 +441,7 @@ class ToolchainTest(EnhancedTestCase):
         self.assertEqual(tc.get_variable('MPIF77'), 'mpif77')
         self.assertEqual(tc.get_variable('MPIF90'), 'mpif90')
 
-        tc = tc_class(version="4.1.13")
+        tc = self.get_toolchain("ictce", version="4.1.13")
         opts = {'usempi': True, 'openmp': True}
         tc.set_options(opts)
         tc.prepare()
